<?php

namespace Drupal\taxonomy_widget_plus\Element;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element\FormElement;
use Drupal\Core\Url;
use Drupal\taxonomy_widget_plus\TaxonomyTreeBuilder;

/**
 * Provides a form element for taxonomy term tree.
 *
 * @FormElement("taxonomy_tree")
 */
class TaxonomyTree extends FormElement {

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    return [
      '#theme' => 'taxonomy_tree',
      '#input' => TRUE,
      '#tree' => TRUE,
      '#process' => [
        [
          static::class,
          'processGroup',
        ],
        [
          static::class,
          'processTaxonomyTree',
        ],
      ],
      '#pre_render' => [
        [
          static::class,
          'preRenderGroup',
        ],
        [
          static::class,
          'preRenderTaxonomyTree',
        ],
      ],
      '#default_value' => [],
      '#attached' => [
        'library' => [
          'taxonomy_widget_plus/taxonomy_widget_plus',
        ],
      ],
    ];
  }

  /**
   * Prepare the attributes for taxonomy tree wrapper.
   */
  public static function preRenderTaxonomyTree($element) {
    $element['#theme_wrappers']['container']['#id'] = $element['#id'] . '--wrapper';

    $add_new_link_id = $element['add_new_link']['#id'];
    $add_new_wrapper_id = $element['add_new']['#id'];
    $element['#attached']['drupalSettings']['taxonomyTreeWidget']['addNewButtonMapping'][$add_new_link_id] = $add_new_wrapper_id;

    return $element;
  }

  /**
   * Processes a taxonomy tree form element.
   */
  public static function processTaxonomyTree(&$element, FormStateInterface $form_state, &$complete_form) {
    $value = is_array($element['#value']) ? $element['#value'] : [];
    $allowed = [];
    $filter = FALSE;

    if (!empty($element['#allowed'])) {
      $allowed = $element['#allowed'];
      $filter = TRUE;
    }

    $options_tree = [];
    foreach ($element['#vocabularies'] as $vocabulary) {
      $options = TaxonomyTreeBuilder::createTermHierarchy(0, $vocabulary->id(), $allowed, $filter, '', $value);
      $options_tree = array_merge($options_tree, $options);
    }

    $element['#options_tree'] = $options_tree;
    $element['#options'] = TaxonomyTreeBuilder::convertTermsToOptions($element['#options_tree'], $allowed, $filter);
    $terms = !empty($element['#options_tree']) ? $element['#options_tree'] : [];

    $tree = new \stdClass();
    $tree->children = $terms;
    unset($element['#needs_validation']);
    $element[] = TaxonomyTreeBuilder::buildTreeRoot($element, $tree, $form_state, $value, $element['#max_choices'], [], 1);

    // Provide an "Add new" utility.
    $taxonomy_storage = \Drupal::entityTypeManager()->getStorage('taxonomy_term');
    $parent = array_keys($taxonomy_storage->loadParents(NULL));

    $tree = $taxonomy_storage->loadTree($vocabulary->id());
    $options = ['<' . t('root') . '>'];
    if (empty($parent)) {
      $parent = [0];
    }
    foreach ($tree as $item) {
      $options[$item->tid] = str_repeat('-', $item->depth) . $item->name;
    }

    $element['add_new_link'] = [
      '#type' => 'link',
      '#title' => t('+ Add new'),
      '#url' => Url::fromUserInput('#add-new-term'),
    ];

    $element['add_new'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => 'hidden',
      ],
    ];
    $element['add_new']['term_name'] = [
      '#type' => 'textfield',
      '#title' => t('Name'),
    ];
    $element['add_new']['parent'] = [
      '#type' => 'select',
      '#title' => t('Parent'),
      '#options' => $options,
    ];

    $element['add_new']['submit'] = [
      '#type' => 'submit',
      // @see  https://www.drupal.org/node/2546700#comment-11164091
      '#value' => t('Add @term', ['@term' => $vocabulary->label()]),
      '#button_type' => 'primary',
      '#limit_validation_errors' => [
        array_merge($element['#parents'], ['add_new']),
      ],
      '#submit' => [[self::class, 'addNewAjaxSubmit']],
      '#ajax' => [
        'callback' => [self::class, 'addNewAjaxCallback'],
        'event' => 'click',
        'wrapper' => $element['#id'] . '--wrapper',
      ],
    ];

    $element['#theme_wrappers']['container'] = [];
    $element['#cache']['#tags'][] = 'taxonomy_term_list';

    return $element;
  }

  /**
   * Ajax submit handler for "Add new" button.
   */
  public static function addNewAjaxSubmit(array $form, FormStateInterface $form_state) {
    $parents = $form_state->getTriggeringElement()['#parents'];
    array_pop($parents);
    $field_name = reset($parents);
    $element = $form[$field_name]['widget'];

    $vocabulary = reset($element['#vocabularies']);
    $term_name = $form_state->getValue(array_merge($parents, ['term_name']));
    $parent_term = $form_state->getValue(array_merge($parents, ['parent']));

    if (!empty($term_name)) {
      $taxonomy_storage = \Drupal::entityTypeManager()->getStorage('taxonomy_term');
      $new_term = $taxonomy_storage->create([
        'name' => $term_name,
        'vid' => $vocabulary->id(),
        'parent' => $parent_term,
      ]);
      $new_term->save();
      $form_state->set('new_term_id', $new_term->id());
    }

    $form_state->setRebuild(TRUE);
  }

  /**
   * Callback for "Add new" button.
   */
  public static function addNewAjaxCallback(array $form, FormStateInterface $form_state) {
    $parents = $form_state->getTriggeringElement()['#parents'];
    array_pop($parents);
    $field_name = reset($parents);
    $element = $form[$field_name]['widget'];

    $new_term_id = $form_state->get('new_term_id');
    $new_term_wrapper_id = $element['#id'];
    $element['#attached']['drupalSettings']['taxonomyTreeWidget']['newTermId'] = $new_term_id;
    $element['#attached']['drupalSettings']['taxonomyTreeWidget']['newTermWrapperId'] = $new_term_wrapper_id;

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function valueCallback(&$element, $input, FormStateInterface $form_state) {
    $value = [];
    $element += [
      '#default_value' => [],
    ];

    foreach ($element['#default_value'] as $key) {
      if (isset($key['target_id'])) {
        $value[$key['target_id']] = $key['target_id'];
      }
    }

    return $value;
  }

}
