<?php

namespace Drupal\taxonomy_widget_plus\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\taxonomy\Entity\Vocabulary;

/**
 * Plugin implementation of the 'taxonomy_widget_plus' widget.
 *
 * @FieldWidget(
 *   id = "taxonomy_widget_plus",
 *   label = @Translation("Taxonomy Widget Plus"),
 *   field_types = { "entity_reference" },
 *   multiple_values = TRUE
 * )
 */
class TaxonomyWidgetPlus extends WidgetBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'sidebar' => TRUE,
      'start_collapsed' => FALSE,
      'use_scrollbar' => TRUE,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public static function isApplicable(FieldDefinitionInterface $field_definition) {
    $storage = $field_definition->getFieldStorageDefinition();
    return $storage->getSetting('target_type') === 'taxonomy_term';
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::settingsForm($form, $form_state);

    $form['sidebar'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Place field in sidebar'),
      '#description' => $this->t('If checked, this field will be placed in the sidebar of the entity form.'),
      '#default_value' => $this->getSetting('sidebar'),
    ];

    $form['start_collapsed'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Collapsed by default'),
      '#description' => $this->t('Show the tree as collapsed when first load.'),
      '#default_value' => $this->getSetting('start_collapsed'),
    ];

    $form['use_scrollbar'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use scrollbar'),
      '#description' => $this->t("Display a vertical scroll when there is a long list of terms."),
      '#default_value' => $this->getSetting('use_scrollbar'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];

    if ($this->getSetting('sidebar')) {
      $summary[] = $this->t('Placed in sidebar');
    }

    if ($this->getSetting('start_collapsed')) {
      $summary[] = $this->t('Start collapsed');
    }

    if ($this->getSetting('use_scrollbar')) {
      $summary[] = $this->t('Use scrollbar for long list');
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $handler_settings = $this->getFieldSetting('handler_settings');
    $vocabularies = Vocabulary::loadMultiple($handler_settings['target_bundles']);

    $element['#type'] = 'taxonomy_tree';
    $element['#title_display'] = 'invisible';
    $element['#default_value'] = $items->getValue();
    $element['#vocabularies'] = $vocabularies;
    $element['#max_choices'] = $this->fieldDefinition->getFieldStorageDefinition()->getCardinality();
    $element['#value_key'] = 'target_id';
    $element['#start_collapsed'] = $this->getSetting('start_collapsed');
    $element['#use_scrollbar'] = $this->getSetting('use_scrollbar');

    $start_collapsed = $this->getSetting('start_collapsed');
    $element['#theme_wrappers'] = [
      'details' => [
        '#attributes' => [
          'open' => !$start_collapsed,
        ],
        // @see https://www.drupal.org/project/drupal/issues/2998194
        '#summary_attributes' => [],
      ],
    ];

    $sidebar = $this->getSetting('sidebar');
    if ($sidebar) {
      $element['#group'] = 'advanced';
      $element['#weight'] = 100;
    }

    return $element;
  }

  /**
   * {@inheritDoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    $flattened_values = [];

    // Processing for radio buttons.
    if ($this->fieldDefinition->getFieldStorageDefinition()->getCardinality() === 1) {
      $selected = $values['widget'];
      $flattened_values[$selected] = $selected;
    }
    else {
      // @todo figure out a more reliable massaging method.
      if (isset($values[0]) && is_array($values[0])) {
        array_walk_recursive($values[0], function ($value, $key) use (&$flattened_values) {
          $flattened_values[$key] = $value;
        });
      }
      else {
        $flattened_values = $values;
      }

      $flattened_values = array_filter($flattened_values, 'is_numeric', ARRAY_FILTER_USE_KEY);
      $flattened_values = array_filter($flattened_values);
    }

    return $flattened_values;
  }

}
