<?php

namespace Drupal\taxonomy_widget_plus;

use Drupal\Component\Utility\Html;

/**
 * Provides a form element for taxonomy term tree.
 *
 * @FormElement("taxonomy_tree")
 */
class TaxonomyTreeBuilder {

  /**
   * Returns a taxonomy term hierarchy in a nested array.
   */
  public static function createTermHierarchy($tid, $vid, &$allowed, $filter, $label, $default = []) {
    $terms = self::getTermChildren($tid, $vid);
    $result = [];

    if ($filter != '') {
      foreach ($allowed as $k => $v) {
        if (array_key_exists($k, $terms)) {
          $term =& $terms[$k];
          $children = self::createTermHierarchy($term->tid, $vid, $allowed, $filter, $label, $default);
          if (is_array($children)) {
            $term->children = $children;
            $term->children_selected = self::hasSelectedChiren($term, $default);
          }
          else {
            $term->children_selected = FALSE;
          }
          array_push($result, $term);
        }
      }
    }
    else {
      foreach ($terms as &$term) {
        if ($filter == '' || array_key_exists($term->tid, $allowed)) {
          $children = self::createTermHierarchy($term->tid, $vid, $allowed, $filter, $label, $default);
          if (is_array($children)) {
            $term->children = $children;
            $term->children_selected = self::hasSelectedChiren($term, $default);
          }
          else {
            $term->children_selected = FALSE;
          }
          array_push($result, $term);
        }
      }
    }

    return $result;
  }

  /**
   * Converts a list of terms to a list of ptions.
   */
  public static function convertTermsToOptions(&$terms, &$allowed, $filter) {
    $options = [];

    if (is_array($terms) && count($terms) > 0) {
      foreach ($terms as $term) {
        if (!$filter || (is_array($allowed) && $allowed[$term->tid])) {
          $options[$term->tid] = $term->name;
          $options += self::convertTermsToOptions($term->children, $allowed, $filter);
        }
      }
    }

    return $options;
  }

  /**
   * Builds a tree root starting from an element.
   */
  public static function buildTreeRoot($element, $term, $form_state, $value, $max_choices, $parent_tids, $depth) {
    $leaves_only = FALSE;
    $container = [
      '#theme' => 'taxonomy_tree_root',
      '#max_choices' => $max_choices,
      '#leaves_only' => $leaves_only,
      '#depth' => $depth,
    ];

    foreach ($term->children as $child) {
      $container[$child->tid] = self::buildTreeItem($element, $child, $form_state, $value, $max_choices, $parent_tids, $container, $depth);
    }

    return $container;
  }

  /**
   * Builds a single item of the taxonomy tree.
   */
  public static function buildTreeItem($element, $term, $form_state, $value, $max_choices, $parent_tids, $parent, $depth) {
    $leaves_only = FALSE;

    $langcode = \Drupal::languageManager()->getCurrentLanguage()->getId();
    $t = NULL;
    if (\Drupal::moduleHandler()->moduleExists('locale') && !empty($term->tid)) {
      $t = \Drupal::entityTypeManager()
        ->getStorage('taxonomy_term')
        ->load($term->tid);

      if ($t && $t->hasTranslation($langcode)) {
        $term_name = $t->getTranslation($langcode)->label();
      }
    }
    if (empty($term_name)) {
      $term_name = $term->name;
    }

    $container = [
      '#theme' => 'taxonomy_tree_item',
      '#max_choices' => $max_choices,
      '#leaves_only' => $leaves_only,
      '#term_name' => $term_name,
      '#depth' => $depth,
    ];

    if (empty($element['#leaves_only']) || count($term->children) == 0) {
      $e = [
        '#type' => ($max_choices == 1) ? 'radio' : 'checkbox',
        '#title' => $term_name,
        '#on_value' => $term->tid,
        '#off_value' => 0,
        '#return_value' => $term->tid,
        '#parent_values' => $parent_tids,
        '#default_value' => isset($value[$term->tid]) ? $term->tid : NULL,
        '#attributes' => isset($element['#attributes']) ? $element['#attributes'] : NULL,
        '#ajax' => isset($element['#ajax']) ? $element['#ajax'] : NULL,
      ];

      if (is_array($e)) {
        if ($e['#type'] == 'radio') {
          $parents_for_id = array_merge($element['#parents'], [$term->tid]);
          $e['#id'] = Html::getId('edit-' . implode('-', $parents_for_id));
          $e['#parents'] = array_merge($element['#parents'], ['widget']);
        }
      }
    }

    $container[$term->tid] = $e;
    if (property_exists($term, 'children') && count($term->children) > 0) {
      $parents = $parent_tids;
      $parents[] = $term->tid;
      $container[$term->tid . '-children'] = self::buildTreeRoot($element, $term, $form_state, $value, $max_choices, $parents, $depth + 1);
    }

    return $container;
  }

  /**
   * Check if this term has a selected child.
   */
  private static function hasSelectedChiren($term, $default) {
    foreach ($term->children as $child) {
      if (isset($default[$child->tid]) || $child->children_selected) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Gets the children list of a term.
   */
  private static function getTermChildren(int $tid, string $vid) {
    $query = \Drupal::database()->select('taxonomy_term_field_data', 't');
    $query->join('taxonomy_term__parent', 'p', 't.tid = p.entity_id');
    $query->fields('t', ['tid', 'name']);
    $query->addField('t', 'vid', 'vocabulary_machine_name');
    $query
      ->condition('t.vid', $vid)
      ->condition('p.parent_target_id', $tid)
      ->addTag('term_access')
      ->addTag('translatable')
      ->orderBy('t.weight')
      ->orderBy('t.name');

    $result = $query->execute();
    $terms = [];

    while ($term = $result->fetchObject()) {
      $terms[$term->tid] = $term;
    }

    return $terms;
  }

}
